import fileutils
from project import Project
import build_constants
import os
import xml.etree.ElementTree as ET


def os_system(cmd, message_on_error=''):
    print cmd
    result = os.system(cmd)
    if result != 0:
        if message_on_error:
            print message_on_error
        exit(-1)
    return True


def get_app_kind_suffix():
    if not Project.instance.with_pro_version:
        return ''
    else:
        return '.' + Project.instance.arg_parser.app_kind


def set_values(string, configuration='', scheme=''):
    string = string.replace('@{ROOT}', fileutils.root_dir)
    string = string.replace('@{APPLE_APP_ID}', Project.instance.apple_id)
    string = string.replace('@{APPLE_USER}', Project.instance.apple_auth_user)
    string = string.replace('@{APPLE_PASSWORD}', Project.instance.apple_auth_password)
    string = string.replace('@{APPLE_TEAM_ID}', Project.instance.apple_team_id)
    string = string.replace('@{CONFIGURATION}', configuration)
    string = string.replace('@{SCHEME}', scheme)
    string = string.replace('@{APP_KIND_SUFFIX}', get_app_kind_suffix())
    string = string.replace('@{XCODEPROJECT}', Project.instance.arg_parser.xcodeproj)
    return string


def windows_build(configuration):
    cmd = set_values(build_constants.win_cmd_build,
                     configuration=configuration
                     )
    os_system(cmd)


def android_change_version(packagename, app_version, build_version):
    path = fileutils.root_dir + '/proj.android{}/app/AndroidManifest.xml'.format(get_app_kind_suffix())
    ET.register_namespace('android', 'http://schemas.android.com/apk/res/android')
    try:
        handle = open(path, 'r')
        tree = ET.parse(handle)
        version = '{}.{}'.format(app_version, build_version) if build_version else app_version
        if build_version and str(build_version) != '0':
            tree.getroot().attrib["{http://schemas.android.com/apk/res/android}versionCode"] = str(build_version)
        tree.getroot().attrib["{http://schemas.android.com/apk/res/android}versionName"] = version
        tree.getroot().attrib["package"] = packagename
        tree.write(path, encoding='utf-8', xml_declaration=True)
    except:
        print 'cannot open file', path
        exit(-1)


def android_build(configuration):
    cmd = set_values(build_constants.android_cmd_build,
                     configuration=configuration
                     )
    os_system(cmd)


def android_upload_apk():
    cmd = set_values(build_constants.android_cmd_upload_apk)
    os_system(cmd)


def ios_change_version(packagename, app_version, build_version, app_kind):
    sh = '/usr/libexec/Plistbuddy'
    plist = 'Info.plist' if app_kind is None else 'info.%s.plist' % app_kind
    plist = '{}/proj.ios/ios/{}'.format(fileutils.root_dir, plist)
    version = '{}.{}'.format(app_version, build_version) if build_version else app_version
    os_system('{} -c "Set CFBundleShortVersionString {}" "{}"'.format(sh, version, plist))
    os_system('{} -c "Set CFBundleIdentifier {}" "{}"'.format(sh, packagename, plist))
    if build_version and str(build_version) != '0':
        os_system('{} -c "Set CFBundleVersion {}" "{}"'.format(sh, build_version, plist))


def ios_build_archive(scheme):
    build = set_values(build_constants.ios_cmd_build, scheme=scheme)
    os_system(build)


def ios_export_archive():
    export_create = set_values(build_constants.ios_export_plist)
    export = set_values(build_constants.ios_cmd_export)
    os_system(export_create)
    os_system(export)


def ios_upload_archive(scheme):
    upload = set_values(build_constants.ios_upload_shell, scheme=scheme)
    os_system(upload)


def ios_upload_inapps():
    upload_inapps = set_values(build_constants.ios_cmd_upload_inapps)
    os_system(upload_inapps)



import version
import arguments
import inapps
import build
import os
import sys
from project import Project
import fileutils
import validate
import compress_images
import create_sublime_project
import generate_android_project
import generate_xcode_project
import locales
import hockeyapp
from analytics.main import analytics


def console():
    main()


def _version():
    print version.__version__


def main():
    parser = arguments.Parser(sys.argv)
    command = parser.command

    fileutils.root_dir = os.path.abspath(parser.root)
    if not os.path.isdir(fileutils.root_dir):
        print 'Invalid path to project [{}]'.format(parser.root)
        exit(-1)

    if command == 'init':
        project = Project(parser, empty=True)
        project.create()
        exit(0)
    if command == 'version':
        print version.__version__
        exit(0)
    if command.startswith('an_'):
        analytics(command, parser)
        exit(0)

    Project.instance = Project(parser)

    if command == 'sublime':
        create_sublime_project.run()
    elif command == 'inapps_download':
        inapps.download_ios_metadata(Project.instance.apple_id)
    elif command == 'inapps':
        inapps.run(Project.instance.package,
                   Project.instance.name,
                   Project.instance.version,
                   Project.instance.gg_inapps,
                   parser)
    elif command == 'build':
        build.run(Project.instance.package,
                  Project.instance.version,
                  parser)
    elif command == 'gen-android':
        generate_android_project.run(parser)
    elif command == 'gen-ios':
        generate_xcode_project.run(parser)
    elif command == 'upload':
        build.upload(parser)
    elif command == 'upload_inapps':
        build.upload_inapps(parser)
    elif command == 'validate':
        validate.run()
    elif command == 'compress':
        compress_images.run(parser)
    elif command == 'locales':
        locales.download(Project.instance.gg_locales)
    elif command == 'create_locales_table':
        locales.convert_to_table()
    elif command == 'hockeyapp_upload':
        hockeyapp.hockey_app_upload(Project.instance, parser)
    else:
        print 'Unknown command [{}]'.format(command)


if __name__ == '__main__':
    if len(sys.argv) == 1:
        sys.argv.extend('validate -r /work/td_core/projects/syndicate-4/'.split(' '))
    main()



import re


class Table:

    def __init__(self):
        self.columns = []
        self.rows = []

    def log(self):
        print '\t'.join(self.columns)
        for line in self.rows:
            print '\t'.join(line)

    def save(self, file):
        file = open(file, 'w')
        file.write('\t'.join(self.columns) + '\n')
        for line in self.rows:
            file.write('\t'.join(line) + '\n')

    def select(self, column):
        if column in self.columns:
            column = self.columns.index(column)
        else:
            print 'table have not parameter [%s]' % column
            return []
        result = []
        for line in self.rows:
            result.append(line[column])
        return result

    def count_if(self, column, equal):
        count = 0
        index = self.columns.index(column)
        for row in self.rows:
            value = row[index]
            if eval(equal.format(value)):
                count += 1
        return count

    def remove_if(self, column, equal):
        rows = []
        index = self.columns.index(column)
        for row in self.rows:
            value = row[index]
            if not eval(equal.format(value)):
                rows.append(row)
        self.rows = rows

    def max(self, column):
        values = self.select(column)
        max_value = -9999
        for value in values:
            if value.isdigit():
                max_value = max(max_value, int(value))
        return max_value


def parse_params(params):
    result = {}
    pairs = params.split(';')
    for pair in pairs:
        name = pair.split(':')[0].strip()
        value = pair.split(':')[1].strip()
        result[name] = value
    return result


def parse(file):
    table = []
    columns = []
    lines = open(file).readlines()
    lines = lines[1:]
    for line in lines:
        l = line.find('{')
        r = line.rfind('}')
        if l != -1 and r != -1:
            params = line[l + 1:r]
            params = parse_params(params)
            table.append(params)
            for name in params:
                if name not in columns:
                    columns.append(name)

    result_table = Table()
    result_table.columns = columns
    for line in table:
        nline = []
        for name in columns:
            value = line[name] if name in line else ''
            nline.append(value)
        result_table.rows.append(nline)
    return result_table


def parse_all(file):
    tables = {}
    columns = []
    lines = open(file).readlines()
    lines = lines[1:]
    for line in lines:
        event = re.findall(r'"[\w\s,:]+",\d+.(\w+)', line)
        if event:
            event = event[0]
        if event not in tables:
            tables[event] = []
        l = line.find('{')
        r = line.rfind('}')
        if l != -1 and r != -1:
            params = line[l + 1:r]
            params = parse_params(params)
            tables[event].append(params)
            for name in params:
                if name not in columns:
                    columns.append(name)

    result = {}
    for event in tables:
        result[event] = Table()
        result[event].columns = columns
        for line in tables[event]:
            nline = []
            for name in columns:
                value = line[name] if name in line else ''
                nline.append(value)
            result[event].rows.append(nline)
    return result

if __name__ == '__main__':
    file = 'Syndicate3-(2017-08-08T02_37_10.527Z-2017-09-07T02_37_10.527Z)-(PurchaseSuccessful)-eventsLog.csv'
    parse(file).log()



import fileutils
import os


class _Cache:
    units = []
    units_path = {}


def get_units_list():
    if _Cache.units:
        return _Cache.units
    units = []
    all_files = []
    dirs = [
        '/Resources/ini/units/creep/',
        '/Resources/ini/units/tower/',
        '/Resources/ini/units/hero/',
        '/Resources/ini/units/',
        ]
    for dir_ in dirs:
        if os.path.isdir(fileutils.root_dir + dir_):
            files = fileutils.getFilesList(fileutils.root_dir + dir_, False)
            all_files.extend([fileutils.root_dir + dir_ + x for x in files])
    # if os.path.isdir(fileutils.root_dir + '/Resources/ini/units/tower/'):
    #     all_files.extend(fileutils.getFilesList(fileutils.root_dir + '/Resources/ini/units/tower/', False))
    # if os.path.isdir(fileutils.root_dir + '/Resources/ini/units/hero/'):
    #     all_files.extend(fileutils.getFilesList(fileutils.root_dir + '/Resources/ini/units/hero/', False))
    # all_files.extend(fileutils.getFilesList(fileutils.root_dir + '/Resources/ini/units/', False))
    for file in all_files:
        name = file
        name = name[name.rindex('/')+1:]
        if name.startswith('_'):
            continue
        if not name.endswith('.xml'):
            continue
        name = name[0: -4]
        if name in ['death', 'death2']:
            continue
        units.append(name)
        _Cache.units_path[name] = file
    _Cache.units = units
    return units


def get_units_path():
    get_units_list()
    return _Cache.units_path



from setuptools import setup, find_packages
from os.path import join, dirname
import stereo7.version

setup(
    name='stereo7',
    version=stereo7.__version__,
    packages=find_packages(),
    long_description=open(join(dirname(__file__), 'README.txt')).read(),
    install_requires=[
        'google-api-python-client',
        'oauth2client',
        'httplib2',
        'pillow',
        'slackclient'
    ],
    entry_points={
        'console_scripts':
            ['stereo7 = stereo7.core:console']
    },
    test_suite='tests'
)



import xml.etree.ElementTree as ET
import game
import re
import os
import fileutils
from project import Project

logs = []
UNITS = {}


def log(msg):
    logs.append(msg)


def validate_unit_events():

    valid_events = [
        [re.compile(r'on_create'), []],
        [re.compile(r'on_see.+'), []],
        [re.compile(r'on_lose.+'), []],
        [re.compile(r'show_.+'), []],
        [re.compile(r'on_shoot_critical'), []],
        [re.compile(r'on_shoot_critical_byangle\d+'), []],
        [re.compile(r'on_shoot'), []],
        [re.compile(r'on_shoot\d+'), []],
        [re.compile(r'on_shoot_byangle\d+'), []],
        [re.compile(r'on_shoot\d+_byangle\d+'), []],
        [re.compile(r'on_sleep'), []],
        [re.compile(r'on_cocking'), []],
        [re.compile(r'on_relaxation'), []],
        [re.compile(r'on_readyfire'), []],
        [re.compile(r'on_charging'), []],
        [re.compile(r'on_charging_\d+'), []],
        [re.compile(r'on_waittarget'), []],
        [re.compile(r'on_waittarget_\d+'), []],
        [re.compile(r'on_waittarget_finish'), []],
        [re.compile(r'on_waittarget_finish_\d+'), []],
        [re.compile(r'on_move'), []],
        [re.compile(r'on_stop'), []],
        [re.compile(r'on_die'), []],
        [re.compile(r'on_die\d+'), []],
        [re.compile(r'on_die_finish'), []],
        [re.compile(r'on_enter'), []],
        [re.compile(r'on_enter_finish'), []],
        [re.compile(r'on_damaged'), ['tower', 'hero', 'creep']],
        [re.compile(r'on_damage'), ['bullet', 'skill']],
        [re.compile(r'turn_\d+'), []],
        [re.compile(r'on_rotate\d+'), []],
        [re.compile(r'on_spawn'), []],
        [re.compile(r'on_spawn_unit_\d+'), []],
        [re.compile(r'on_healing'), []],
        [re.compile(r'crit_area_\d+'), ['hero']],
        [re.compile(r'crit_longrange_\d+'), ['hero']],
        [re.compile(r'crit_area'), ['hero']],
        [re.compile(r'crit_longrange'), ['hero']],
        [re.compile(r'on_select'), ['hero']],
        [re.compile(r'on_deselect'), ['hero']],

        # custom events
        [re.compile(r'spawn:\d'), ['desant']],
        [re.compile(r'on_jumping_\d'), ['creep']],
        [re.compile(r'on_on_ground'), ['creep']],
        [re.compile(r'on_rage_initiator_rand\d+'), ['creep']],
        [re.compile(r'kill_teleport'), ['hero']],
        [re.compile(r'kill_teleport_back'), ['hero']],
        [re.compile(r'teleport'), ['hero']],
        [re.compile(r'create_rocket_l'), ['hero']],
        [re.compile(r'create_rocket_r'), ['hero']],

        # robot
        [re.compile(r'on_builded'), ['hero']],
        [re.compile(r'on_skill_activated_robot_skill_ballista'), ['hero']],
        [re.compile(r'on_skill_activated_robot_skill_minigun'), ['hero']],
        [re.compile(r'on_skill_activated_robot_skill_rocket'), ['hero']],
        [re.compile(r'on_skill_activated_robot_skill_sniper'), ['hero']],
        [re.compile(r'on_skill_activated_robot_skill_tesla'), ['hero']],
        [re.compile(r'on_skill_prepare_robot_skill_ballista'), ['hero']],
        [re.compile(r'on_skill_prepare_robot_skill_minigun'), ['hero']],
        [re.compile(r'on_skill_prepare_robot_skill_rocket'), ['hero']],
        [re.compile(r'on_skill_prepare_robot_skill_sniper'), ['hero']],
        [re.compile(r'on_skill_prepare_robot_skill_tesla'), ['hero']],
        [re.compile(r'on_skill_top'), ['hero']],
        [re.compile(r'on_skill_down'), ['hero']],
        [re.compile(r'prepare_gun'), ['hero']],
        [re.compile(r'prepare_turret'), ['hero']],
        [re.compile(r'skill_gun'), ['hero']],
        [re.compile(r'skill_turret'), ['hero']],
    ]

    mandatory_events = [
        []
    ]

    for unit, path in UNITS.items():
        type = get_unit_type(unit)
        if type is None:
            continue
        events = get_all_unit_events(unit)
        for event in events:
            valid = False
            for pattern_info in valid_events:
                pattern = pattern_info[0]
                types = pattern_info[1]
                match = re.match(pattern, event)
                if match is not None:
                    valid = not types or type in types
                    break
            if not valid:
                log('Unknown unit event [%s:%s][%s]' % (unit, type, event))


def get_unit_type(unit):
    global UNITS
    if not UNITS:
        UNITS = game.get_units_path()

    path = UNITS[unit]
    while True:
        if not os.path.isfile(path):
            log('Template not exist [%s]'%path)
            break
        root = ET.parse(path).getroot()
        if 'unittype' in root.attrib:
            return root.attrib['unittype']
        if 'template' in root.attrib:
            path = fileutils.root_dir + '/Resources/' + root.attrib['template']
        else:
            return None


def get_all_unit_events(unit):
    global UNITS
    if not UNITS:
        UNITS = game.get_units_path()

    result = []
    path = UNITS[unit]
    while True:
        root = ET.parse(path).getroot()

        events_xml_range = root.findall('events')
        for events_xml in events_xml_range:
            for child in events_xml:
                name = child.attrib['name']
                result.append(name)

        if 'template' in root.attrib:
            path = fileutils.root_dir + '/Resources/' + root.attrib['template']
        else:
            break
    return result


def validate():
    # try:
    validate_unit_events()
    # except Exception as e:
    #     print(e)
    #     exit(-1)
